/*
** Module   : MenuChuiImpl.java
** Abstract : CHUI client implementation of Menu.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
*/
package com.goldencode.p2j.ui.client.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.border.*;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * <code>Menu</code> CHUI client implementation class. 
 */
public class MenuChuiImpl
extends Menu<ChuiOutputManager>
{
   /** Border around the panel. */
   private Border<ChuiOutputManager> border = null;
   
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public MenuChuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (MenuConfig) cfg);
      
      MenuConfig mcfg = (MenuConfig) cfg;
      if (mcfg.popupOnly)
      {
         border = new LineBorder();
      }
   }
   
   /**
    * Drawing worker method.
    */
   public void drawPanel()
   {
      screen().clear(this);

      if (border != null)
         border.drawBorder(this, dimension());
   }

   /**
    * Draw the peer container. The methods calls the draw() method for
    * ScrollPane instance for the peer container. In addition the required
    * clipping rectangle is calculated for ChildFrame containers because they
    * must be rendered inside the parent container.
    * <p>
    * Drawing for the redirected terminal differs. Labels/headers are drawn
    * only once per page, instead of the usual once per menu.
    */
   @Override
   public void draw()
   {
      drawPanel();
      super.draw();
      //this.firstFocus();
      //this.setFocus(this.focus());
      //EventManager.postEvent(new FocusEvent(this.focus(), EventType.FOCUS_GAINED));
   }

   /**
    * Hide/show widget.
    * <p>
    * If the menu is being shown and not yet attached to a window, attached it to the default
    * window.
    * 
    * @param   visible
    *          New visible state.
    */
   @Override
   public void setVisible(boolean visible)
   {
      setDisplayed(visible);
      
      if (visible && parent() == null)
      {
         setParent((Container<ChuiOutputManager>) Window.getDefaultWindow());
      }
      
      for(Widget<ChuiOutputManager> w : widgets())
      {
         w.setVisible(true);
      }

      super.setVisible(visible);
   }

   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    */
   @Override
   public Dimension dimension()
   {
      // TODO take note about toggle boxes and submenus. Probably in menu-item or in sub-menu.
      double width = 0; 
      for(Widget<?> w : widgets())
      {
         AbstractWidget<?> aw = (AbstractWidget<?>) w;
         if (width < aw.width())
         {
            width = aw.width(); 
         }
      }
      
      return new Dimension(width + 2, widgets().length + 2);
   }

   /**
    * Get widget location relative to container.
    * 
    * @return  relative widget location.
    */
   @Override
   public Point location()
   {
      if (config().popupOnly)
      {
         Widget<?> owner = screen().getRegistry().getComponent(config().ownerId);
         if (owner == null)
            return new Point(0, 0); // owner is not yet specified
         
         AbstractWidget<?> awOwner = (AbstractWidget<?>) owner;
         return new Point(awOwner.location().x + 1, awOwner.location().y + awOwner.height() + 1);
      }
      
      return new Point(0, 0); // menubar
   }
}
