/*
** Module   : MenuChuiImpl.java
** Abstract : CHUI client implementation of Menu.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
*/
package com.goldencode.p2j.ui.client.chui;

import java.util.List;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.border.*;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * <code>Menu</code> CHUI client implementation class. 
 */
public class SubMenuChuiImpl
extends SubMenu<ChuiOutputManager>
{
   /** Border around the panel. */
   private Border<ChuiOutputManager> border = null;
   
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public SubMenuChuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (SubMenuConfig) cfg);
      
      border = new LineBorder();
   }
   
   /**
    * Drawing worker method.
    */
   public void drawPanel()
   {
      screen().clear(this);

      if (border != null)
         border.drawBorder(this, dimension());
   }

   /**
    * Draw the peer container. The methods calls the draw() method for
    * ScrollPane instance for the peer container. In addition the required
    * clipping rectangle is calculated for ChildFrame containers because they
    * must be rendered inside the parent container.
    * <p>
    * Drawing for the redirected terminal differs. Labels/headers are drawn
    * only once per page, instead of the usual once per menu.
    */
   @Override
   public void draw()
   {
      if (config().displayed)
      {
         drawPanel();
         super.draw();
      }
      else
      {
         // cp from MenuItemChuiImpl.draw()
         AbstractContainer<ChuiOutputManager> parent = (AbstractContainer<ChuiOutputManager>) parent();
         List<Widget<ChuiOutputManager>> widgets = parent.getWidgetsList();
         int idx = widgets.indexOf(this);
         if (idx == -1) 
         {
            throw new IllegalStateException("MENU-ITEM is not contained in its parents widgets list");
         }
         Point p = parent.location();
         p = new Point(p.x + 1, p.y + 1 + idx);
         screen().at(p).append(this.config().title + " ->", Color.NORMAL);
      }
   }

   /**
    * Hide/show widget.
    * <p>
    * If the menu is being shown and not yet attached to a window, attached it to the default
    * window.
    * 
    * @param   visible
    *          New visible state.
    */
   @Override
   public void setVisible(boolean visible)
   {
      // SUB-MENU is not displayed (dropped down) by default
      // setDisplayed(visible);
      
      for(Widget<ChuiOutputManager> w : widgets())
      {
         w.setVisible(true);
      }

      super.setVisible(visible);
   }

   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    */
   @Override
   public Dimension dimension()
   {
      // TODO take note about toggle boxes and submenus. Probably in menu-item or in sub-menu.
      double width = 0; 
      for(Widget<?> w : widgets())
      {
         AbstractWidget<?> aw = (AbstractWidget<?>) w;
         if (width < aw.width())
         {
            width = aw.width(); 
         }
      }
      
      return new Dimension(width + 2, widgets().length + 2);
   }

   /**
    * Get widget location relative to container.
    * 
    * @return  relative widget location.
    */
   @Override
   public Point location()
   {
      /*
      AbstractContainer<?> parent = (AbstractContainer<?>) this.parent();
      int idx = parent.getWidgetsList().indexOf(this);
      */
      
      // TODO
      return super.location();
      /*
      Widget<?> owner = screen().getRegistry().getComponent(config().ownerId);
      if (owner == null)
         return new Point(0, 0); // owner is not yet specified
      
      AbstractWidget<?> awOwner = (AbstractWidget<?>) owner;
      return new Point(awOwner.location().x + 1, awOwner.location().y + awOwner.height() + 1);
      */
   }

   /**
    * Get widget width.
    * 
    * @return  widget width.
    */
   @Override
   public double width()
   {
      if (!config().displayed)
         return config().title.length() + 3; // " ->" will be added
      else
         // TODO process this case like MENU ?
         return super.width();
   }

   /**
    * Get widget height.
    * 
    * @return  widget height.
    */
   @Override
   public double height()
   {
      if (!config().displayed)
         return 1;
      else
         // TODO process this case like MENU ?
         return super.height();
   }
}
